==> nui_animation-0.0.1.tar.gz: lib/nui_button_animation.dart <==
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

class NUIBouncingAnimation extends StatefulWidget {
  final Widget child;
  final VoidCallback onTap;
  final VoidCallback onLongPress;
  final double scaleFactor;
  final Duration duration;

  const NUIBouncingAnimation({
    Key key,
    @required this.child,
    @required this.onTap,
    this.onLongPress,
    this.scaleFactor = 1,
    this.duration = const Duration(milliseconds: 200),
  }) : super(key: key);

  @override
  _NUIBouncingAnimationState createState() => _NUIBouncingAnimationState();
}

class _NUIBouncingAnimationState extends State<NUIBouncingAnimation> with SingleTickerProviderStateMixin {
  //// Animation controller
  AnimationController _controller;

  double _scale;
  GlobalKey _childKey = GlobalKey();
  bool _isOutside = false;
  Widget get child => widget.child;
  VoidCallback get onPressed => widget.onTap;
  VoidCallback get onLongPressed => widget.onLongPress;
  double get scaleFactor => widget.scaleFactor;
  Duration get duration => widget.duration;

  /// We instantiate the animation controller
  /// The idea is to call setState() each time the controller's
  /// value changes
  @override
  void initState() {
    _controller = AnimationController(
      vsync: this,
      duration: duration,
      lowerBound: 0.0,
      upperBound: 0.1,
    )..addListener(() {
      setState(() {});
    });
    super.initState();
  }

  /// Dispose the animation controller
  @override
  void dispose() {
    _controller?.dispose();
    super.dispose();
  }

  /// Each time the [_controller]'s value changes, build() will be called
  /// We just have to calculate the appropriate scale from the controller value
  /// and pass it to our Transform.scale widget
  @override
  Widget build(BuildContext context) {
    _scale = 1 - (_controller.value * scaleFactor);
    return GestureDetector(
      onTapDown: _onTapDown,
      onTapUp: _onTapUp,
      onTapCancel: (){ _onTapUp(null, cancelled: true);},
      onLongPress: _onLongPress,
      onLongPressEnd: (details) => _onLongPressEnd(details, context),
//      onHorizontalDragEnd: _onDragEnd,
//      onVerticalDragEnd: _onDragEnd,
//      onHorizontalDragUpdate: (details) => _onDragUpdate(details, context),
//      onVerticalDragUpdate: (details) => _onDragUpdate(details, context),
      child: Transform.scale(
        key: _childKey,
        scale: _scale,
        child: child,
      ),
    );
  }

  /// Simple method called when we need to notify the user of a press event
  _triggerOnPressed() {
    if (onPressed != null) {
      onPressed();
    }
  }

  _triggerOnLongPressed(){
    if(onLongPressed != null){
      onLongPressed();
    }
  }

  /// We start the animation
  _onTapDown(TapDownDetails details) {
    _controller.forward();
  }

  /// We reverse the animation and notify the user of a press event
  _onTapUp(TapUpDetails details, {bool cancelled: false}) {
    print("NUIButtonAnimation -> On tap up");
    Future.delayed(duration, () {
      _controller.reverse();
    });

    if(cancelled != true) {
      _triggerOnPressed();
    }
  }

  /// Here we are listening on each change when drag event is triggered
  /// We must keep the [_isOutside] value updated in order to use it later
  _onDragUpdate(DragUpdateDetails details, BuildContext context) {
    final Offset touchPosition = details.globalPosition;
    _isOutside = _isOutsideChildBox(touchPosition);
  }

  /// When this callback is triggered, we reverse the animation
  /// If the touch position is inside the children renderBox, we notify the user of a press event
  _onLongPressEnd(LongPressEndDetails details, BuildContext context) {
    final Offset touchPosition = details.globalPosition;

    if (!_isOutsideChildBox(touchPosition)) {
      if(onLongPressed == null) {
        _triggerOnPressed();
      }
    }

    if(onLongPressed == null) {
      _controller.reverse();
    }
  }

  _onLongPress(){
    if(onLongPressed != null) {
      _controller.reverse();
      _triggerOnLongPressed();
    }
  }

  /// When this callback is triggered, we reverse the animation
  /// As we do not have position details, we use the [_isOutside] field to know
  /// if we need to notify the user of a press event
  _onDragEnd(DragEndDetails details) {
    if (!_isOutside) {
      _triggerOnPressed();
    }
    _controller.reverse();
  }

  /// Method called when we need to now if a specific touch position is inside the given
  /// child render box
  bool _isOutsideChildBox(Offset touchPosition) {
    final RenderBox childRenderBox = _childKey.currentContext.findRenderObject();
    final Size childSize = childRenderBox.size;
    final Offset childPosition = childRenderBox.localToGlobal(Offset.zero);

    return (touchPosition.dx < childPosition.dx ||
        touchPosition.dx > childPosition.dx + childSize.width ||
        touchPosition.dy < childPosition.dy ||
        touchPosition.dy > childPosition.dy + childSize.height);
  }
}

==> nui_animation-1.0.1.tar.gz: lib/nui_button_animation.dart <==
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

class NUIBouncingAnimation extends StatefulWidget {
  final Widget child;
  final VoidCallback onTap;
  final VoidCallback? onLongPress;
  final double scaleFactor;
  final Duration duration;

  const NUIBouncingAnimation({
    Key? key,
    required this.child,
    required this.onTap,
    this.onLongPress,
    this.scaleFactor = 1,
    this.duration = const Duration(milliseconds: 200),
  }) : super(key: key);

  @override
  _NUIBouncingAnimationState createState() => _NUIBouncingAnimationState();
}

class _NUIBouncingAnimationState extends State<NUIBouncingAnimation> with SingleTickerProviderStateMixin {
  //// Animation controller
  late AnimationController _controller;

  double? _scale;
  GlobalKey _childKey = GlobalKey();
  bool _isOutside = false;
  Widget get child => widget.child;
  VoidCallback get onPressed => widget.onTap;
  VoidCallback? get onLongPressed => widget.onLongPress;
  double get scaleFactor => widget.scaleFactor;
  Duration get duration => widget.duration;

  /// We instantiate the animation controller
  /// The idea is to call setState() each time the controller's
  /// value changes
  @override
  void initState() {
    _controller = AnimationController(
      vsync: this,
      duration: duration,
      lowerBound: 0.0,
      upperBound: 0.1,
    )..addListener(() {
      setState(() {});
    });
    super.initState();
  }

  /// Dispose the animation controller
  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  /// Each time the [_controller]'s value changes, build() will be called
  /// We just have to calculate the appropriate scale from the controller value
  /// and pass it to our Transform.scale widget
  @override
  Widget build(BuildContext context) {
    _scale = 1 - (_controller.value * scaleFactor);
    return GestureDetector(
      onTapDown: _onTapDown,
      onTapUp: _onTapUp,
      onTapCancel: (){ _onTapUp(null, cancelled: true);},
      onLongPress: _onLongPress,
      behavior: HitTestBehavior.translucent,
      onLongPressEnd: (details) => _onLongPressEnd(details, context),
//      onHorizontalDragEnd: _onDragEnd,
//      onVerticalDragEnd: _onDragEnd,
//      onHorizontalDragUpdate: (details) => _onDragUpdate(details, context),
//      onVerticalDragUpdate: (details) => _onDragUpdate(details, context),
      child: Transform.scale(
        key: _childKey,
        scale: _scale,
        child: child,
      ),
    );
  }

  /// Simple method called when we need to notify the user of a press event
  _triggerOnPressed() {
    if (onPressed != null) {
      onPressed();
    }
  }

  _triggerOnLongPressed(){
    if(onLongPressed != null){
      onLongPressed!();
    }
  }

  /// We start the animation
  _onTapDown(TapDownDetails details) {
    _controller.forward();
  }

  /// We reverse the animation and notify the user of a press event
  _onTapUp(TapUpDetails? details, {bool cancelled = false}) {
    print("NUIButtonAnimation -> On tap up");
    Future.delayed(duration, () {
      _controller.reverse();
    });

    if(cancelled != true) {
      _triggerOnPressed();
    }
  }

  /// Here we are listening on each change when drag event is triggered
  /// We must keep the [_isOutside] value updated in order to use it later
  _onDragUpdate(DragUpdateDetails details, BuildContext context) {
    final Offset touchPosition = details.globalPosition;
    _isOutside = _isOutsideChildBox(touchPosition);
  }

  /// When this callback is triggered, we reverse the animation
  /// If the touch position is inside the children renderBox, we notify the user of a press event
  _onLongPressEnd(LongPressEndDetails details, BuildContext context) {
    final Offset touchPosition = details.globalPosition;

    if (!_isOutsideChildBox(touchPosition)) {
      if(onLongPressed == null) {
        _triggerOnPressed();
      }
    }

    if(onLongPressed == null) {
      _controller.reverse();
    }
  }

  _onLongPress(){
    if(onLongPressed != null) {
      _controller.reverse();
      _triggerOnLongPressed();
    }
  }

  /// When this callback is triggered, we reverse the animation
  /// As we do not have position details, we use the [_isOutside] field to know
  /// if we need to notify the user of a press event
  _onDragEnd(DragEndDetails details) {
    if (!_isOutside) {
      _triggerOnPressed();
    }
    _controller.reverse();
  }

  /// Method called when we need to now if a specific touch position is inside the given
  /// child render box
  bool _isOutsideChildBox(Offset touchPosition) {
    final RenderBox childRenderBox = _childKey.currentContext!.findRenderObject() as RenderBox;
    final Size childSize = childRenderBox.size;
    final Offset childPosition = childRenderBox.localToGlobal(Offset.zero);

    return (touchPosition.dx < childPosition.dx ||
        touchPosition.dx > childPosition.dx + childSize.width ||
        touchPosition.dy < childPosition.dy ||
        touchPosition.dy > childPosition.dy + childSize.height);
  }
}
